@@ -2,6 +2,7 @@ import re
 
 from django.contrib.auth import get_user_model
 from django.urls.exceptions import Resolver404, NoReverseMatch
+from django.utils.functional import cached_property
 from django.utils.translation import gettext as _
 from ipware import get_client_ip
 from jwcrypto import jwt, jwe
@@ -24,38 +25,23 @@ class BaseAuthentication(authentication.BaseAuthentication):
     def authenticate(self, request):
         self.request = request
 
-    @property
-    def _is_verification_endpoint(self):
-        from xauth.accounts.views import AccountViewSet
+    @cached_property
+    def _auth_view(self):
+        return get_class(f"{AUTH_APP_LABEL}.views", "AccountViewSet")(request=self.request)
 
-        view = AccountViewSet(request=self.request)
+    def _is_request_from_any_of(self, url_names):
         try:
-            endpoint = view.reverse_action(view.verify_account.url_name, kwargs=view.request.resolver_match.kwargs)
+            endpoints = [
+                self._auth_view.reverse_action(url_name, kwargs=self._auth_view.request.resolver_match.kwargs)
+                for url_name in url_names
+            ]
         except (Resolver404, NoReverseMatch):
             return False
-        return endpoint == self.request_url_in_lowercase
+        return self.request_url_in_lowercase in endpoints
 
     @property
     def _is_activation_endpoint(self):
-        from xauth.accounts.views import AccountViewSet
-
-        view = AccountViewSet(request=self.request)
-        try:
-            endpoint = view.reverse_action(view.activate_account.url_name, kwargs=view.request.resolver_match.kwargs)
-        except (Resolver404, NoReverseMatch, AttributeError):
-            return False
-        return endpoint == self.request_url_in_lowercase
-
-    @property
-    def _is_password_reset_endpoint(self):
-        from xauth.accounts.views import AccountViewSet
-
-        view = AccountViewSet(request=self.request)
-        try:
-            endpoint = view.reverse_action(view.reset_password.url_name, kwargs=view.request.resolver_match.kwargs)
-        except (Resolver404, NoReverseMatch):
-            return False
-        return endpoint == self.request_url_in_lowercase
+        return self._is_request_from_any_of([self._auth_view.activate_account.url_name])
 
     def _get_wrapped_response(self, request, user):
         if user.is_active or self._is_activation_endpoint:
@@ -92,11 +78,25 @@ class JWTTokenAuthentication(BaseAuthentication):
         try:
             token = get_class(f"{AUTH_APP_LABEL}.token.generator", "Token")(None)
             claims = token.get_claims(token=jwt_token)
-            if claims["sub"] != "verification" and self._is_verification_endpoint:
+            if claims["sub"] == "verification" and (
+                not self._is_request_from_any_of(
+                    [
+                        self._auth_view.verify_account.url_name,
+                        self._auth_view.request_verification_code.url_name,
+                    ]
+                )
+            ):
                 raise jwe.JWException
-            elif claims["sub"] != "activation" and self._is_activation_endpoint:
+            elif claims["sub"] == "password-reset" and (
+                not self._is_request_from_any_of(
+                    [
+                        self._auth_view.reset_password.url_name,
+                        self._auth_view.request_temporary_password.url_name,
+                    ]
+                )
+            ):
                 raise jwe.JWException
-            elif claims["sub"] != "password-reset" and self._is_password_reset_endpoint:
+            elif claims["sub"] == "activation" and (not self._is_activation_endpoint):
                 raise jwe.JWException
             else:
                 try:
